/**
 * FLL - Level 3
 *
 * Project: Example
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common define types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _example_common_define_h
#define _example_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * example_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_example_allocation_d_
  #define example_allocation_console_d 4
  #define example_allocation_large_d   256
  #define example_allocation_pipe_d    16384
  #define example_allocation_small_d   16
#endif // _di_example_allocation_d_

/**
 * Flags passed to the main function or program.
 *
 * When number mode is not specified, then mode is "string" mode (there is no flag for "string" mode).
 *
 * example_main_flag_*_d:
 *   - none:                   No flags set.
 *   - copyright:              Print copyright.
 *   - error:                  Check if status is "error".
 *   - fine:                   Check if status is "fine".
 *   - help:                   Print help.
 *   - pipe:                   Use the input pipe.
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 *   - warning:                Check if status is "warning".
 */
#ifndef _di_example_main_flag_d_
  #define example_main_flag_none_d                   0x0
  #define example_main_flag_copyright_d              0x1
  #define example_main_flag_error_d                  0x2
  #define example_main_flag_fine_d                   0x4
  #define example_main_flag_help_d                   0x8
  #define example_main_flag_pipe_d                   0x10
  #define example_main_flag_version_d                0x20
  #define example_main_flag_version_copyright_help_d 0x29
  #define example_main_flag_warning_d                0x40
#endif // _di_example_main_flag_d_

/**
 * Flags for fine-tuned print control.
 *
 * example_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_example_print_flag_d_
  #define example_print_flag_none_d    0x0
  #define example_print_flag_debug_d   0x1
  #define example_print_flag_error_d   0x2
  #define example_print_flag_file_d    0x4
  #define example_print_flag_in_d      0x8
  #define example_print_flag_out_d     0x10
  #define example_print_flag_message_d 0x20
  #define example_print_flag_warning_d 0x40
#endif // _di_example_print_flag_d_

/**
 * The program signal defines.
 *
 * example_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_example_signal_d_
  #define example_signal_check_d          500000
  #define example_signal_check_failsafe_d 20000
  #define example_signal_check_tiny_d     4
  #define example_signal_check_short_d    16
#endif // _di_example_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _example_common_define_h
